<?php
#CMS - CMS Made Simple
#(c)2008 by Nicholas Vrtis (vrtisworks@users.sf.net)
#This project's homepage is: http://www.vrtisworks.com/cmsms/
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function smarty_cms_function_passwordmaker($params, &$smarty) {

    if (isset($_POST['len'])) {
        $len=sprintf('%d',$_POST['len']);
        if ($len<3) {
            $len=3;
        }
    } else {
        $len=8;
    }
    if (isset($_POST['nbrspcl'])) {
        $nbrspcl=sprintf('%d',$_POST['nbrspcl']);
        if ($nbrspcl<0) {
            $nbrspcl=3;
        }
        if ($nbrspcl>$len) {
            $nbrspcl=$len;
        }
    } else {
        $nbrspcl=3;
    }
?>
  <form action="<?php $_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING'] ?>" method="post" name="passwordmakerForm">
<div class="formbox">
    <div class="formrow">
      <span class="formlabel">Length:</span>
      <input type="text" class="forminput" size="3" name="len" value="<?php echo $len;?>" />
    </div>
    <div class="formrow">
      <span class="formlabel">Number special chars:</span>
     <input type="text" class="forminput" size="3" name="nbrspcl" value="<?php echo $nbrspcl;?>" />
    </div>
    <div class="formrow">
        <input type="submit" class="formsubmit" name="doit" value="Generate" />
    </div>
</div>
 </form>
<br />
<table class="wordsbox">
<?php
    //require_once '../passwordmaker/WordBox.php';
    $subs= array('a'=>'*','b'=>'6','c'=>'(','d'=>'-','e'=>'3','f'=>'4','g'=>'9','h'=>'H','i'=>'1','j'=>'/','k'=>'<','l'=>'7',
        'm'=>'>','n'=>'#','o'=>'0','p'=>'+','q'=>'?','r'=>'^','s'=>'$','t'=>'2','u'=>'_','v'=>'5','w'=>':','x'=>'!','y'=>'?','z'=>'@');
    $wordbox=new WordBox();
    for ($i=0; $i<10; $i++) {
        $baseword=$wordbox->generate($len);
        $password=$baseword;
        if ($nbrspcl>0) {
            $n=$nbrspcl;
            while ($n>0) {
                $idx=rand(1,$len)-1;
                if ($baseword[$idx]==$password[$idx]) {
                    //If they are still == then we haven't done a substitution yet.
                    //Flip a coin heads=Make it a capital, tails=substitute
                    if (rand(0,1)==0) {
                        //Substitue a special character
                        $password[$idx]=$subs[$password[$idx]];
                    } else {
                        //Make it a capital
                        $password[$idx]=strtoupper($password[$idx]);
                    }
                    $n--;
                }
            }
        }
        if ($nbrspcl==0) {
            //No special processing.. just do the one version...
            echo '<tr><td>',htmlentities($baseword, ENT_QUOTES),'</td></tr>',"\n";
        } else {
            echo '<tr><td>',htmlentities($baseword, ENT_QUOTES),'</td><td>',htmlentities($password, ENT_QUOTES),'</td></tr>',"\n";
        }
    }
    echo '</table>';
}

function smarty_cms_help_function_passwordmaker() {
	?>
	<h3>What does this do?</h3>
	<p>Generates a list of semi-pronouncable passwords, with and without special character substitutions.</p>
	<h3>How do I use it?</h3>
	<p>Just insert the tag into your template/page like: <code>{passwordmaker}</code><br>
	<p>The following CSS classes are used:
	<ul>
	    <li><em>formbox</em> The outer DIV that marks the parameter form (where they can
	       change the defaults for the list of passwords that gets generated.</li>
        <li><em>formrow</em> Each row in the parameter form.</li>
        <li><em>formlabel</em> The label part of each parameter.</li>
        <li><em>forminput</em> The input part of each parameter.</li>
        <li><em>formsubmit</em> The Submit button.</li>
        <li><em>wordsbox</em> The table where the resulting generated passwords are put</li>
	</ul>
	</p>
	<h3>What parameters does it take?</h3>
	<ul>
		<li>None</li>
	</ul>
	</p>
	<?php
}

function smarty_cms_about_function_passwordmaker() {
	?>
	<p>Author: Nicholas Vrtis&lt;vrtisworks@users.sourceforge.net&gt;</p>
	<p>Version: 1.0</p>
	<p>
	Change History:<br/>
	None
	</p>
	<?php
}

class WordBox {

    var $catalog;     //The input file of probabilities
    var $table;       //We will read the probabilities table into here...
    var $prefixSum;
    var $record = -1;

    function WordBox() {
    }

    function prefixSumCalc() {
        $sum=0;
        for ($i=0; $i<26; $i++){
            for ($j=0; $j<26; $j++) {
                $sum += $this->probability($i,$j,26);
            }
        }
        return $sum;
    }

    function infixSum($i, $j) {
        $sum=0;
        for ($k=0; $k<26; $k++) {
            $sum += $this->probability($i,$j,$k);
        }
        return $sum;
    }

    function suffixSum($i, $j){
        $sum=0;
        for ($k=0; $k<26; $k++) {
            if ($this->probability($i,$j,$k)!=0) {
                $sum += $this->probability($j,$k,27);
            }
        }
        return $sum;
    }

  /* private int random(int range) // range may be 1..65536
  {
    randomSeed = ((randomSeed * 32145) + 1) & 0x0FFFF;
    return (randomSeed * range) >>> 16;
  } */

    function probability($i, $j, $k) {
        if ($this->record != $i) { // cache table
            $offset=$i*728+288;
            fseek($this->catalog,$offset);
			$this->table=fread($this->catalog,728);
    	    $this->record = $i;
        }
        $value = ord(substr($this->table,($j*28+$k),1));
        return $value;
    }

    function setDatabase() {
    	global $gCms;
        if ($this->catalog == null) {
            $this->catalog = fopen($gCms->config['uploads_path'].
                '/passwordmaker/ENGLISH.PDB','rb');
      	    $this->prefixSum = $this->prefixSumCalc(); // calculate only once
        }
        return true;
    }

    function generate($wordLength) {

        if (!$this->setDatabase()) {
    	    return 'No Database';
        }

        $word = '';
        $length = 0;
        $chra=ord('A');

        while ($length < $wordLength) {
            //select_prefix:
            while (true) { // select a word prefix (2 characters)
                $sigma = rand(0,$this->prefixSum);
                $sum = 0;
                $i = 0;
                while ($i < 26) {
                    $j = 0;
                    while ($j < 26) {
                        $sum += $this->probability($i,$j,26);
                        if ($sum > $sigma) {
                            if ($length++ < $wordLength) $word.=chr($chra+$i);
                            if ($length++ < $wordLength) $word.=chr($chra+$j);
                            break 3;
                        }
 	                    $j++;
   	                }
     	            $i++;
       	        }
            }
 	        //select_next:
   	        while ($length < $wordLength) { // select more characters
                if ($length == $wordLength-1 && $length > 1) { // select a suffix
       	            $sigma = $this->suffixSum($i,$j);
         	        if ($sigma != 0) { // is there a possible continuation?
           	            $sigma = rand(0,$sigma);
                        $sum = 0;
 	                    for ($k=0; $k<26; $k++) {
   	                        // does this make an existing trigram?
     	                    if ($this->probability($i,$j,$k) != 0) {
                                $sum += $this->probability($j,$k,27);
                            }
       	                    if ($sum > $sigma) {
         	                    $word.=chr($chra+$k);
           	                    $length++;
             	                break 2;
                            }
 	                    }
   	                }
     	        }
       	        $sigma = $this->infixSum($i,$j);
                if ($sigma == 0) {
                    break; // is this a dead-end?
                }
                $sigma = rand(0,$sigma);
 	 	        $sum = 0;
     	        //select_trigram:
       	        for ($k=0; $k<26; $k++) {
         	        $sum += $this->probability($i,$j,$k);
	                if ($sum > $sigma) {
                        $word.=chr($chra+$k);
   	                    $length++;
     	                $i=$j; $j=$k;
       	                break;
         	        }
	            }
            }
   	    }
    	return strtolower($word);
    }
}
//========================================================================================
//
// WORDBOX - Generate random pronounceable words based on trigram probabilities
//
// WordBox is Copyright (c) 1999 by Klaus Rindtorff
//
// Permission to use, copy, and distribute this software and its documentation
// for any purpose, without fee, and without a written agreement is hereby
// granted, provided that the above copyright notice and this paragraph and
// the following two paragraphs appear in all copies.
//
// IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
// SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
// AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
// BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
// UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// WORDBOX generates random pronounceable words based on trigram probabilities.
// Each word is generated randomly based on the probability of individual trigrams in
// a given language. The probabilities for each language are stored in a database.
//
// Each database consists of 26 tables for the starting character 'A'..'Z'. Each table
// contains 26 rows for the middle character, and each row contains 28 entries for the
// third character 'A'..'Z' and two extra values. Each entry gives the probability for
// the occurence of the respective trigram. The first extra value gives the probability
// that the relevant character pair is a word prefix. The second extra value gives the
// probability that the character pair is a word suffix. All probabilities are relative
// and all values are normalized to be in the range 0 to 255.
//
// The trigram statistics were generated from word lists for the following languages:
//
//   ENGLISH          153418 words
// Version 1.00  1998-12-4   Klaus Rindtorff, klaus@ivorytower.com
// Version 1.10  1999-01-19  minor modifications, added comments, added Names database
// Version 1.20  1999-07-11  changes for Waba Beta8 and wextras 1.01
//                           modified wextras List class to compile with Waba Beta8
// Version 1.30  1999-12-28  changes for Waba Beta9 and wextras 1.21
//                           added Japanese names
//
// Version 2.00  2006-11-19  Converted to php by vrtis at vrtisworks dot com
//========================================================================================

?>
